# Converts an mod to obj, and obj to mod
# Note:
#   * Your mesh should be all tri's.
#   * Only vertex and face information is used

class ArmagetronModelConverter
  FILETYPES = ["obj", "mod"].freeze
  attr_reader :extension
  
  def initialize(file_to_convert)
    @file_to_convert = file_to_convert
    @vertexes = []
    @faces = []
    @extension = @file_to_convert.split(".").last.downcase
    
    determine_file_to_convert_filetype    
  end
  
  def determine_file_to_convert_filetype
    if @extension == "obj"
      convert_obj_to_mod
    elsif @extension == "mod"
      convert_mod_to_obj
    else
      raise "Unknown filetype. Acceptable types (case does not matter): #{FILETYPES.join(", ")}"
    end
  end
  
  def convert_mod_to_obj
    open(@file_to_convert,'r') do |file|
      file.each_line { |line| convert_mod_to_obj_line(line) }
    end
  end
  
  def convert_obj_to_mod
    open(@file_to_convert,'r') do |file|
      file.each_line { |line| convert_obj_to_mod_line(line) }
    end
  end
  
  # An OBJ face:
  #   f 1/1 2/2 3/3
  #   We are only interested in the number before the slash, the second number is the texture point.
  #   Sometimes and OBJ face is like this:
  #   f 1 2 3
  #
  # A MOD face:
  #   f 1 2 3
  #
  # An OBJ vertex:
  #   v 1 2 3
  #
  # A MOD vertex:
  #   v (vertex number) 1 2 3
  #
  def convert_mod_to_obj_line(line)
    line = line.split
    
    case line[0]
      when /v/
        @vertexes << "v #{line[2]} #{line[3]} #{line[4]}\n"
        
      when /f/
        @faces << "f #{line[1]} #{line[2]} #{line[3]}\n"
    end  
  end
  
  def convert_obj_to_mod_line(line)
    line = line.split
    
    case line[0]
      when /v/
        @vertexes << "v #{@vertexes.size+1} #{line[1]} #{line[2]} #{line[3]}\n"
        
      when /f/
        @faces << "f #{split_face_obj_to_mod(line[1])} #{split_face_obj_to_mod(line[2])} #{split_face_obj_to_mod(line[3])}\n" 
    end  
  end
  
  def split_face_obj_to_mod(face)
    face.split("/")[0]
  end
  
  def to_s
    mod_string = ""
    @vertexes.each { |vertex| mod_string << vertex }
    @faces.each    { |face|   mod_string << face   }
    return mod_string
  end
  
  # IO sucks.
  def save
    if @extension == "obj"
      ext = "mod"
    else
      ext = "obj"
    end
    
    filename = File.basename(@file_to_convert).split(".")
    filename[-1] = nil
    filename.flatten!
    filename = filename.join(".")
    filename << ext
    directory = File.dirname(@file_to_convert)
    File.join(directory, filename)
    
    open(File.join(directory, filename), 'w') do |file|
      file.print(to_s)
    end
  end
end